package com.sun.dft.glassfish.http.failover.runtime;

import java.io.IOException;
import java.util.StringTokenizer;

import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.sun.dft.glassfish.config.BaseTestCase;
import com.sun.dft.glassfish.helpers.FailureInjectionService;
import com.sun.dft.glassfish.utils.TestUtils;
import com.sun.dft.glassfish.helpers.HttpMessageLogger;


public class HttpFailoverRuntimeTest34 extends BaseTestCase {

    private String servingInstance, newServingInstance, thirdServingInstance;
	
    @BeforeTest
    protected void setUp() throws Exception {
		asadminHelper.deployWithLBReconfig("web/runtime/Session.war", "Session", TestUtils.getDefaultClusterName());
	        checkPoint("Deployed Session");

    }

    @Test(description="Storing and retrieving Session with varying sizes across failover ")
    public void RuntimeTest1() throws Exception {
		String webserverHostPort = webserverHelper.getHost() + ":" + webserverHelper.getHttpPort();
		String request1 = "/Session/SessionData";
		String requestString = "http://" + webserverHostPort + request1;

		checkPoint("** Sending Request to Webserver " + webserverHostPort + "> " + request1);
		WebConversation webConversation = new WebConversation();
		WebRequest webRequest = new GetMethodWebRequest(requestString);
		WebResponse response = webConversation.getResponse(webRequest);
		logHttpRequestResponse(response);  // response for request1
                HttpMessageLogger.logHttpRequestResponse(webConversation, webRequest, response);


                boolean sessionContainsRetrievedData = response.getText().contains("sessions is new :  true");
		checkPoint("** Session data retrieved " + sessionContainsRetrievedData);

                boolean secondSessionContainsRetrievedData = response.getText().contains("session data size : 2000");
		checkPoint("** Session is Invalidated " + secondSessionContainsRetrievedData);

		Assert.assertTrue(sessionContainsRetrievedData && secondSessionContainsRetrievedData,
				"Session data present");
		servingInstance = findServingInstance(response);
		checkPoint("** Serving Instance (1st Request): " + servingInstance);

		checkPoint("** Killing serving instance...");
		new FailureInjectionService().killInstance(servingInstance);

                // TODO: Use TestUtils to wait for message that indicates unhealthy instance.
		TestUtils.sleepSeconds(10);

		String request2 = "/Session/SessionData?dataname=user1&numChars=10240";
		String newRequestString = "http://" + webserverHostPort + request2;

		checkPoint("** Sending Request to Webserver " + webserverHostPort + "> " + request2);
		WebRequest newWebRequest = new GetMethodWebRequest(newRequestString);
		WebResponse newResponse = webConversation.getResponse(newWebRequest);
		logHttpRequestResponse(newResponse);  // response for request2
                HttpMessageLogger.logHttpRequestResponse(webConversation, newWebRequest, newResponse);

		newServingInstance = findServingInstance(newResponse);
		checkPoint("** Serving Instance (2nd Request): " + newServingInstance);

                
		Assert.assertTrue(! newServingInstance.equals(servingInstance), 
				"Serving instance expected to be different:" + servingInstance + "; " + newServingInstance);
		
		//sessionContainsRetrievedData = newResponse.getText().contains("sessions is new :  false");
		//checkPoint("** Session data retrieved " + sessionContainsRetrievedData);

                secondSessionContainsRetrievedData = newResponse.getText().contains("session data size : 10240");
		checkPoint("** session data size : 10240 " + secondSessionContainsRetrievedData);

                boolean thirdSessionContainsRetrievedData = newResponse.getText().contains("user1 =");
		checkPoint("** user1 = " + thirdSessionContainsRetrievedData );

                boolean fourthSessionContainsRetrievedData = newResponse.getText().contains("session data size : 2000");
		checkPoint("** session data size : 2000 " + fourthSessionContainsRetrievedData);


		Assert.assertTrue(secondSessionContainsRetrievedData && thirdSessionContainsRetrievedData && fourthSessionContainsRetrievedData  ,
				"Session data retrieved");

                String request3 = "/Session/SessionData?dataname=user3&numChars=10240";
		String thirdRequestString = "http://" + webserverHostPort + request3;

		checkPoint("** Sending Request to Webserver " + webserverHostPort + "> " + request3);                

                WebRequest thirdWebRequest = new GetMethodWebRequest(thirdRequestString);
		WebResponse thirdResponse = webConversation.getResponse(thirdWebRequest);
		logHttpRequestResponse(thirdResponse);  // response for request3
                HttpMessageLogger.logHttpRequestResponse(webConversation, thirdWebRequest, thirdResponse);

		thirdServingInstance = findServingInstance(thirdResponse);
		checkPoint("** Serving Instance (3rd Request): " + thirdServingInstance);
                //sessionContainsRetrievedData = thirdResponse.getText().contains("sessions is new :  false");
		//checkPoint("** Session data retrieved " + sessionContainsRetrievedData);

                secondSessionContainsRetrievedData = thirdResponse.getText().contains("session data size : 10240");
		checkPoint("** session data size : 10240 " + secondSessionContainsRetrievedData);

                thirdSessionContainsRetrievedData = thirdResponse.getText().contains("user1 =");
		checkPoint("** user1 = " + thirdSessionContainsRetrievedData );

                fourthSessionContainsRetrievedData = thirdResponse.getText().contains("session data size : 2000");
		checkPoint("** session data size : 2000 " + fourthSessionContainsRetrievedData);

                boolean fifthSessionContainsRetrievedData = thirdResponse.getText().contains("user3 =");
		checkPoint("** user3 = " + fifthSessionContainsRetrievedData);

                Assert.assertTrue(secondSessionContainsRetrievedData && thirdSessionContainsRetrievedData && fourthSessionContainsRetrievedData && fifthSessionContainsRetrievedData ,
				"Session data retrieved");

    }

    


    // TODO: Check: This is perhaps better done by checking server logs ? That way it will be reusable ?
    private String  findServingInstance(WebResponse response) throws IOException {
    	String responseText = response.getText();
    	StringTokenizer tokenizer = new StringTokenizer(responseText, "\n");
    	while (tokenizer.hasMoreTokens()){
    		String token = tokenizer.nextToken();
    		if (token.contains("Executed From Instance:")){
    			String instanceName = token.substring(token.indexOf("<b>")+3, token.indexOf("</b>"));
    			logger.info("Serving Instance: " + instanceName);
    			return instanceName;
    		}
    	}
    	String errorMessage = "Unable to find serving Instance from the response text";
    	logger.severe(errorMessage);
		throw new RuntimeException(errorMessage);
	}

	public static synchronized void logHttpRequestResponse(WebResponse response) throws IOException {
        System.out.println("-------------Request/Response : START ------------");
        System.out.println(response);
        System.out.println(response.getText());
        System.out.println("-------------Request/Response : END --------------");
    }

	@AfterTest
	public void tearDown(){
                asadminHelper.startCluster();
		asadminHelper.undeploy("Session", TestUtils.getDefaultClusterName());
	        checkPoint("Undeployed Session");
	}
}
